import 'dart:typed_data';
import '../marker_handler/marker_data.dart';
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_maps/maps.dart';
import 'dart:math';
import 'dart:convert';

/// A class containing thickness data for each subdivision of the map.
class IceThicknessModel {
  IceThicknessModel(this.subDivID, this.thickness);

  final String subDivID;
  final int thickness;
}

/// ChoroplethMap is a stateful widget that contains a choropleth map.
/// The map is created using the Syncfusion Flutter Maps library and
/// coordinates fetched from the server.
class ChoroplethMap extends StatefulWidget {
  const ChoroplethMap({Key? key,
    required this.relation,
    required this.measurements
  }) : super(key: key);

  final Uint8List relation;
  final List<Measurement> measurements;

  @override
  _ChoroplethMapState createState() => _ChoroplethMapState();
}

class _ChoroplethMapState extends State<ChoroplethMap> {
  late MapShapeSource mapShapeSource;
  late final MapZoomPanBehavior _zoomPanBehavior = MapZoomPanBehavior();
  List<IceThicknessModel> iceThicknessList = <IceThicknessModel>[];

  @override
  void initState() {
    super.initState();

    final Random random = Random();
    for (int i = 0; i <= 60; i++) {
      int randomNumber = random.nextInt(21); // 0 -> 20, NB: temp test data
      iceThicknessList.add(IceThicknessModel(i.toString(), randomNumber));
    }
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        SfMaps(
          layers: [
            MapShapeLayer(
              source: MapShapeSource.memory( // Map polygon
                widget.relation, // JSON coordinates from server
                shapeDataField: 'properties.SubDivID',
                dataCount: iceThicknessList.length,
                primaryValueMapper: (int index) => iceThicknessList[index].subDivID,
                shapeColorValueMapper: (int index) => iceThicknessList[index].thickness,
              ),
              color: Colors.lightBlueAccent, // Map color
              zoomPanBehavior: _zoomPanBehavior,
              strokeColor: Colors.orange,
            ),
          ],
        ),
        CustomPaint( // Render polygons clipped on map
          size: Size.infinite,
          painter: OverlayPainter(mapData: widget.relation),
        ),
      ],
    );
  }
}


/// OverlayPainter is a class extending CustomPainter to
class OverlayPainter extends CustomPainter {
  final Uint8List mapData;

  const OverlayPainter({
    required this.mapData, // Pass map shape
  });

  @override
  void paint(Canvas canvas, Size size) {
    // NB: test polygon
    Path testPol = Path();
    testPol.moveTo(60.8015, 10.5908);
    testPol.lineTo(60.5146, 10.8270);
    testPol.lineTo(60.7115, 11.3242);
    testPol.close();

    // Decode map data
    var mapShape = json.decode(utf8.decode(mapData));

    // Extract polygons and render them
    for (var feature in mapShape['features']) {
      var geometry = feature['geometry'];
      var coordinates = geometry['coordinates'];

      for (var ring in coordinates) {
        var points = <Offset>[];
        for (var point in ring) {
          points.add(Offset(point[0], point[1]));
        }
        var path = Path()..addPolygon(points, true);

        // Render each map shape
        canvas.drawPath(path, Paint()..color = Colors.lightBlueAccent);
      }
    }

    // Now clip the rendering to the map shape
    var mapBoundsPath = Path();
    for (var feature in mapShape['features']) {
      var geometry = feature['geometry'];
      var coordinates = geometry['coordinates'];

      for (var ring in coordinates) {
        var points = <Offset>[];
        for (var point in ring) {
          points.add(Offset(point[0], point[1]));
        }
        mapBoundsPath.addPolygon(points, true);
      }
    }
    canvas.clipPath(mapBoundsPath);

    // Render test shape over map, not clipped to map
    canvas.drawPath(testPol, Paint()..color = Colors.pink.shade900);
  }


  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return false;
  }
}
